﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;             //【Debug】
using System.Drawing;                 //【Bitmap】【Color】
using System.Drawing.Imaging;         //【BitmapData】
using System.Runtime.InteropServices; //【Marshal.Copy】
using System.Threading;           //【Threadを使う為に必要】
using Microsoft.Kinect;           //【参照設定】プロジェクト(P)⇒参照の追加(F)⇒.NETタブ⇒Microsoft.Kinectを選択
using Microsoft.DirectX;          //【参照設定】プロジェクト(P)⇒参照の追加(F)⇒参照タブ
using Microsoft.DirectX.Direct3D; // DirectXを使う為
//【ディレクトリ】C:\Windows\Microsoft.NET\DirectX for Managed Code\1.0.2902.0\
//【選択dll】Microsoft.DirectX.dll, Microsoft.DirectX.Direct3D.dll, Microsoft.DirectX.Direct3DX.dll

//****************************************************************************
//【私設研究所Neo-Tech-Lab.com】『KINECT SENSORデモ・プログラム集』
//【プログラム】『3D-Player』【Cyber Space】
//【作者】上田智章
// All Rights reserved.  Copyright 2012  by ☆Tomoaki Ueda☆
//
//【Microsoft Visual Studio 2010 Express】 Visual C# 2010 Express
//【Microsoft KINECT for Windows SDK Version1】
//【使用許諾事項】
//(1)本プログラムの使用、ソースコードの一部または全部の利用によって生じた
//　いかなる不利益に対しても、著者およびＣＱ出版株式会社が補償することは
//　ありません。自己責任の下での使用及び利用を行わなければなりません。
//****************************************************************************
//【内容】Kinect for Windowsは、Frameworks Ver.4.0でのみ動作可能です。
//　ところが、Managed DirectXはFrameworks Ver.4.0上ではデフォルトで動作しない
//  ので、App.configファイルで設定をしてやる必要があります。
//  このプログラムでは、背景画をSurfaceで書き換えてから２つの正方形を描画する
//  サンプル・プログラムです。
//****************************************************************************

namespace CapsuleShapeBody
{
    public partial class Form1 : Form
    {
        //【KINECT関連】-------pictureBox1の大きさ------------------------------------------
        private const int KS_ScreenWidth = 640;  //【KINECTのImageFrame横方向画素数】 
        private const int KS_ScreenHeight = 480; //【KINECTのImageFrame縦方向画素数】

        //【DirectX関連】-------------------------------------------------------------------
        private Device D3Ddevice = null; //【Direct3Dデバイス】
        private Vector3 D3DcameraPosition = new Vector3(-10.0f, 10.0f, -150.0f); //【カメラの位置】
        private Vector3 D3DcameraEyeUp = new Vector3(0.0f, 0.0f, 1.0f);    //【Z軸方向が上】
        private Vector3 D3DcameraTarget = new Vector3(0.0f, 1.0f, 0.0f);   //【注視点座標】
        private Vector3 D3Dlight = new Vector3(-0.57735f, -0.57735f, -0.57735f); //【光源方向】
        private VertexBuffer D3DvtxBuffer = null; //【頂点バッファ】球体の頂点情報(座標と色)を格納するバッファ
        private IndexBuffer D3DidxBuffer = null;  //【インデックスバッファ】球体を構成する三角形の頂点番号を格納するバッファ
        private Surface D3Dsurface;
        private Surface D3Dbackbuffer;

        //【NTL拡張現実センサ関連】---------------------------------------------------------
        private Thread processThread;            //【MainLoop用Thread】
        private bool processRun;                 //Thread実行フラグ
        private string[] bgImageFile = { "../../image/MMD.bmp", "../../image/Sky.bmp", "../../image/Villa1.JPG", "../../image/Villa2.JPG" };
        private int bgImage = 0;

        public Form1()
        {
            InitializeComponent();
            pictureBox1.ClientSize = new Size(KS_ScreenWidth, KS_ScreenHeight);
            D3Ddevice = PrepareD3dDevice(this.pictureBox1); //【DirectX初期化】device
            processRun = true;                              //【Thread実行フラグ】
            processThread = new Thread(MainProcessThread);  //【Main処理ループのためのThreadを定義】
            processThread.Start();                          //【Thread開始】
        }

        // ■■■■■■■■■■　【Direct3Dデバイスを準備する関数】　■■■■■■■■■■
        private Device PrepareD3dDevice(Control i_window)
        {   //【参考URL】http://www.atelier-blue.com/program/mdirectx/3d/3d02-01.htm
            PresentParameters pp = new PresentParameters(); //【プレゼンテーション・パラメータを作成】
            pp.Windowed = true;                             //【ウィンドウモード】pictureBox1を表示域に設定
            pp.SwapEffect = SwapEffect.Discard;             //【スワップエフェクトを設定】
            pp.BackBufferFormat = Format.X8R8G8B8;          //【描画用グラフィックメモリ】RGB各8bit, 1670万色
            pp.BackBufferCount = 1;                         //【バッファの数】1
            pp.BackBufferWidth = KS_ScreenWidth;            //
            pp.BackBufferHeight = KS_ScreenHeight;          //
            pp.EnableAutoDepthStencil = true;               //【Ｚバッファを使用する】
            pp.AutoDepthStencilFormat = DepthFormat.D16;    //【Ｚバッファ16bit】
            CreateFlags fl_base = CreateFlags.FpuPreserve;  //【初期化に必要なフラグ】

            try { return new Device(0, DeviceType.Hardware, i_window.Handle, fl_base | CreateFlags.HardwareVertexProcessing, pp); }
            catch (Exception ex1)
            {
                Debug.WriteLine(ex1.ToString());
                try { return new Device(0, DeviceType.Hardware, i_window.Handle, fl_base | CreateFlags.SoftwareVertexProcessing, pp); }
                catch (Exception ex2)
                {
                    Debug.WriteLine(ex2.ToString());
                    try { return new Device(0, DeviceType.Reference, i_window.Handle, fl_base | CreateFlags.SoftwareVertexProcessing, pp); }
                    catch (Exception ex3) { throw ex3; }
                }
            }
        }

        //***************************************************************
        //【拡張現実センサ用のThread】
        //***************************************************************
        public void MainProcessThread()
        {
            CustomVertex.PositionColored[] NTL_vtxData = new CustomVertex.PositionColored[8]; //8点の情報を格納するためのメモリを確保
            const float CUBE_SIZE = 20.0f;//1辺40[mm]の頂点を設定
            NTL_vtxData[0] = new CustomVertex.PositionColored(-CUBE_SIZE, CUBE_SIZE, CUBE_SIZE, Color.Yellow.ToArgb());
            NTL_vtxData[1] = new CustomVertex.PositionColored(CUBE_SIZE, CUBE_SIZE, CUBE_SIZE, Color.Gray.ToArgb());
            NTL_vtxData[2] = new CustomVertex.PositionColored(-CUBE_SIZE, CUBE_SIZE, -CUBE_SIZE, Color.Purple.ToArgb());
            NTL_vtxData[3] = new CustomVertex.PositionColored(CUBE_SIZE, CUBE_SIZE, -CUBE_SIZE, Color.Red.ToArgb());
            NTL_vtxData[4] = new CustomVertex.PositionColored(-CUBE_SIZE, -CUBE_SIZE, CUBE_SIZE, Color.SkyBlue.ToArgb());
            NTL_vtxData[5] = new CustomVertex.PositionColored(CUBE_SIZE, -CUBE_SIZE, CUBE_SIZE, Color.Orange.ToArgb());
            NTL_vtxData[6] = new CustomVertex.PositionColored(-CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE, Color.Green.ToArgb());
            NTL_vtxData[7] = new CustomVertex.PositionColored(CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE, Color.Blue.ToArgb());
            Int16[] NTL_idxData = new Int16[] { 2, 0, 1, 1, 3, 2, 4, 0, 2, 2, 6, 4, 5, 1, 0, 0, 4, 5, 7, 3, 1, 1, 5, 7, 6, 2, 3, 3, 7, 6, 4, 6, 7, 7, 5, 4 };
            float AngleX=0f, AngleY=0f; //【座標回転角度】

            //立方体（頂点数8）の準備
            D3DvtxBuffer = new VertexBuffer(typeof(CustomVertex.PositionColored), 8, D3Ddevice, Usage.None, CustomVertex.PositionColored.Format, Pool.Managed);
            using (GraphicsStream data = D3DvtxBuffer.Lock(0, 0, LockFlags.None)) //頂点バッファをロックする
            {
                data.Write(NTL_vtxData); // 頂点データを頂点バッファにコピーします
                D3DvtxBuffer.Unlock();// 頂点バッファのロックを解除します
            }
            D3DidxBuffer = new IndexBuffer(D3Ddevice, 12 * 3 * 2, Usage.WriteOnly, Pool.Managed, true);
            using (GraphicsStream data = D3DidxBuffer.Lock(0, 0, LockFlags.None))
            {
                data.Write(NTL_idxData); // インデックスデータをインデックスバッファにコピーします
                D3DidxBuffer.Unlock(); // インデックスバッファのロックを解除します
            }

            D3Dbackbuffer = D3Ddevice.GetBackBuffer(0, 0, BackBufferType.Mono);
            D3Dsurface = D3Ddevice.CreateOffscreenPlainSurface(640, 480, Format.X8R8G8B8, Pool.Default); //画像を格納するサーフェイスを作成
            SurfaceLoader.FromFile(D3Dsurface, bgImageFile[bgImage], Filter.None, 0x000000); //【ファイルから元画像を入れる】

            //********************************************************
            //
            //********************************************************
            while (processRun)
            {
                float Omega = (float)Math.PI / 180.0f;
                Microsoft.DirectX.Matrix m1 = new Matrix();  //【行列】
                Microsoft.DirectX.Matrix m2 = new Matrix();  //【行列】
                Microsoft.DirectX.Matrix m3 = new Matrix();  //【行列】
                
                D3Ddevice.Clear(ClearFlags.Target|ClearFlags.ZBuffer, Color.DarkBlue, 1.0f, 0);

                D3Ddevice.RenderState.ZBufferEnable = true;                 //【陰面処理をＺバッファ方式で行う】
                D3Ddevice.RenderState.ZBufferFunction = Compare.LessEqual;  //【手前がＺ値の小さい側】.LessEqual;    
                D3Ddevice.RenderState.Lighting = false;                     //【光源計算処理を行う】true;
                D3Ddevice.RenderState.CullMode = Cull.CounterClockwise;

                D3Ddevice.BeginScene(); //【シーン開始】

                try { D3Ddevice.StretchRectangle(D3Dsurface, new Rectangle(0, 0, 640, 480), D3Dbackbuffer, new Rectangle(0, 0, 640, 480), TextureFilter.None); }
                catch (Exception em) { Debug.Write(em.ToString()); }

                //【ワールド、ビュー、射影行列を設定】
                m1 = Microsoft.DirectX.Matrix.RotationY(AngleX * Omega);  //【y軸回りに15°回転】
                m1 *= Microsoft.DirectX.Matrix.RotationX(AngleY * Omega); //【x軸回りに25°回転】
                AngleX += 0.5f; if (AngleX > 360.0f) { AngleX -= 360.0f; }
                AngleY += 0.3f; if (AngleX > 360.0f) { AngleY -= 360.0f; }
                D3Ddevice.Transform.World = Microsoft.DirectX.Matrix.Identity;
                //【ビュー変換の設定】(左手座標系ビュー行列で設定する) (0,0,0)から、Z+方向を向いて、上方向がY軸
                D3Ddevice.Transform.View = Microsoft.DirectX.Matrix.LookAtLH(D3DcameraPosition, D3DcameraEyeUp, D3DcameraTarget);
                D3Ddevice.Transform.Projection = Microsoft.DirectX.Matrix.PerspectiveFovLH((float)Math.PI / 4.0f, 1.0f, 0.001f, 1000.0f);
                //【パラメータ】視野角45°, 画面のアスペクト比1.0f, 最も近い距離1.0f, 最も遠い距離100.0f
                //【ビューポート設定】
                Viewport vp = new Viewport();
                vp.X = 0; vp.Y = 0; vp.Height = KS_ScreenHeight; vp.Width = KS_ScreenWidth; vp.MaxZ = 1.0f;
                D3Ddevice.Viewport = vp;

                D3Ddevice.SetStreamSource(0, this.D3DvtxBuffer, 0);
                D3Ddevice.VertexFormat = CustomVertex.PositionColored.Format;
                D3Ddevice.Indices = D3DidxBuffer; // インデックスバッファをセット
                m2 = m1 * Microsoft.DirectX.Matrix.Translation(-30.0f, 95.0f, 230.0f);
                D3Ddevice.SetTransform(TransformType.World, m2); // 計算したマトリックスで座標変換
                D3Ddevice.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, 8, 0, 12);
                m3 = m1 * Microsoft.DirectX.Matrix.Translation(-40.0f, -30.0f, 20.0f);
                D3Ddevice.SetTransform(TransformType.World, m3); // 計算したマトリックスで座標変換
                D3Ddevice.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, 8, 0, 12);
                D3Ddevice.EndScene();
                D3Ddevice.Present();

                Application.DoEvents();
                this.Invalidate();
            }
        }

        //***************************************************************
        //【拡張現実センサ用のThread】
        //***************************************************************
        private void button1_Click(object sender, EventArgs e)
        {
            processRun = false;
            D3Ddevice.Dispose();
            D3DidxBuffer.Dispose();
            D3DidxBuffer.Dispose();
            D3Dbackbuffer.Dispose();
            D3Dsurface.Dispose();
            this.Close();
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        { //【MMD】
            bgImage = 0;
            SurfaceLoader.FromFile(D3Dsurface, bgImageFile[bgImage], Filter.None, 0x000000); //【ファイルから元画像を入れる】
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        { //【Sky】
            bgImage = 1;
            SurfaceLoader.FromFile(D3Dsurface, bgImageFile[bgImage], Filter.None, 0x000000); //【ファイルから元画像を入れる】
        }

        private void radioButton3_CheckedChanged(object sender, EventArgs e)
        { //【桂離宮】
            bgImage = 2;
            SurfaceLoader.FromFile(D3Dsurface, bgImageFile[bgImage], Filter.None, 0x000000); //【ファイルから元画像を入れる】
        }

        private void radioButton4_CheckedChanged(object sender, EventArgs e)
        { //【桂離宮】
            bgImage = 3;
            SurfaceLoader.FromFile(D3Dsurface, bgImageFile[bgImage], Filter.None, 0x000000); //【ファイルから元画像を入れる】
        }


    }
}
